/*
    Generated by sbus code generator

    Copyright (C) 2017 Red Hat

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef _SBUS_IFP_KEYGENS_H_
#define _SBUS_IFP_KEYGENS_H_

#include <talloc.h>

#include "sbus/sbus_request.h"
#include "responder/ifp/ifp_iface/sbus_ifp_arguments.h"

const char *
_sbus_ifp_key_
   (TALLOC_CTX *mem_ctx,
    struct sbus_request *sbus_req);

const char *
_sbus_ifp_key_s_0
   (TALLOC_CTX *mem_ctx,
    struct sbus_request *sbus_req,
    struct _sbus_ifp_invoker_args_s *args);

const char *
_sbus_ifp_key_ssu_0_1_2
   (TALLOC_CTX *mem_ctx,
    struct sbus_request *sbus_req,
    struct _sbus_ifp_invoker_args_ssu *args);

const char *
_sbus_ifp_key_su_0_1
   (TALLOC_CTX *mem_ctx,
    struct sbus_request *sbus_req,
    struct _sbus_ifp_invoker_args_su *args);

const char *
_sbus_ifp_key_u_0
   (TALLOC_CTX *mem_ctx,
    struct sbus_request *sbus_req,
    struct _sbus_ifp_invoker_args_u *args);

#endif /* _SBUS_IFP_KEYGENS_H_ */
